import { AfterViewChecked, EventEmitter, OnInit, AfterViewInit, ElementRef } from '@angular/core';
import { ControlValueAccessor, FormControl } from '@angular/forms';
import { CtrCompleter } from '../directives/completerDirective';
import { CompleterData } from '../services/completerDataService';
import { CompleterService } from '../services/completerService';
import { CompleterItem } from './completerItemComponent';
import 'rxjs/add/operator/catch';
export declare class CompleterCmp implements OnInit, ControlValueAccessor, AfterViewChecked, AfterViewInit {
    private completerService;
    dataService: CompleterData;
    inputName: string;
    inputId: string;
    pause: number;
    minSearchLength: number;
    maxChars: number;
    overrideSuggested: boolean;
    clearSelected: boolean;
    clearUnselected: boolean;
    fillHighlighted: boolean;
    placeholder: string;
    matchClass: string;
    fieldTabindex: number;
    autoMatch: boolean;
    disableInput: boolean;
    inputClass: string;
    autofocus: boolean;
    openOnFocus: boolean;
    initialValue: any;
    autoHighlight: boolean;
    label: string;
    focused: boolean;
    selected: EventEmitter<CompleterItem>;
    highlighted: EventEmitter<CompleterItem>;
    blur: EventEmitter<void>;
    focusEvent: EventEmitter<void>;
    opened: EventEmitter<boolean>;
    keyup: EventEmitter<any>;
    keydown: EventEmitter<any>;
    completer: CtrCompleter;
    ctrInput: ElementRef;
    searchStr: string;
    control: FormControl;
    private displaySearching;
    private displayNoResults;
    private _onTouchedCallback;
    private _onChangeCallback;
    private _focus;
    private _open;
    private _textNoResults;
    private _textSearching;
    constructor(completerService: CompleterService);
    value: any;
    ngAfterViewInit(): void;
    ngAfterViewChecked(): void;
    onTouched(): void;
    writeValue(value: any): void;
    registerOnChange(fn: any): void;
    registerOnTouched(fn: any): void;
    datasource: CompleterData | string | Array<any>;
    textNoResults: string;
    textSearching: string;
    ngOnInit(): void;
    onBlur(): void;
    onFocus(): void;
    onKeyup(event: any): void;
    onKeydown(event: any): void;
    onChange(value: string): void;
    open(): void;
    close(): void;
    focus(): void;
    isOpen(): boolean;
}
